@@ -1,6 +1,6 @@
 /* Interface to file chache API
  *
- * Copyright (C) 2014-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2014-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -26,9 +26,9 @@
 
 extern char *cache_dir;
 
-char *cache_file       (char *name, char *buf, size_t len);
+char *cache_file       (char *name, const char *sysname, char *buf, size_t len);
 int   read_cache_file  (ddns_t *ctx);
-int   write_cache_file (ddns_alias_t *alias);
+int   write_cache_file (ddns_alias_t *alias, const char *name);
 
 #endif /* INADYN_CACHE_H_ */
 
