@@ -41,13 +41,9 @@
 #define DYNDNS_MY_CHECKIP_URL	"/"
 #define DYNDNS_MY_IP_SSL        DDNS_CHECKIP_SSL_UNSUPPORTED
 
-/*
- * 2020-02-18: ipify.org support HTTPS and should be default for new
- * providers, unless the provider has its own checkip infrastructure.
- */
-#define DDNS_MY_IP_SERVER       "api.ipify.org"
-#define DDNS_MY_CHECKIP_URL	"/"
-#define DDNS_MY_IP_SSL          DDNS_CHECKIP_SSL_SUPPORTED
+#define DDNS_MY_IP_SERVER       "ifconfig.me"
+#define DDNS_MY_CHECKIP_URL	"/ip"
+#define DDNS_MY_IP_SSL          DDNS_CHECKIP_SSL_UNSUPPORTED
 
 /* Some default settings */
 #define DDNS_DEFAULT_STARTUP_SLEEP        0       /* sec */
@@ -72,14 +68,14 @@
 #define USERNAME_LEN                      128     /* chars */
 #define PASSWORD_LEN                      256     /* chars */
 #define SERVER_NAME_LEN                   256     /* chars */
-#define SERVER_URL_LEN                    256     /* chars */
+#define SERVER_URL_LEN                    512     /* chars */
 #ifdef INET6_ADDRSTRLEN
 # define MAX_ADDRESS_LEN                  INET6_ADDRSTRLEN
 #else
 # define MAX_ADDRESS_LEN                  46
 #endif
 
-#define MAX_NUM_RESPONSES                 5
+#define MAX_NUM_RESPONSES                 6
 #define MAX_RESPONSE_LEN                  32
 
 typedef enum {
@@ -110,6 +106,7 @@ typedef struct {
 } ddns_name_t;
 
 typedef struct {
+	int            force_addr_update;
 	int            ip_has_changed;
 	char           address[MAX_ADDRESS_LEN];
 
@@ -137,6 +134,9 @@ typedef struct di {
 	char           server_response[MAX_NUM_RESPONSES][MAX_RESPONSE_LEN];
 	size_t         server_response_num;
 
+	/* Interface for IP */
+	char           *ifname;
+
 	/* Address of "What's my IP" checker */
 	ddns_name_t    checkip_name;
 	char           checkip_url[SERVER_URL_LEN];
@@ -190,7 +190,6 @@ typedef struct {
 	int            num_iterations;
 	int            initialized;
 	int            change_persona;
-	int            force_addr_update;
 	int            use_proxy;
 	int            abort;
 
@@ -223,6 +222,7 @@ extern uid_t uid;
 extern gid_t gid;
 
 int ddns_main_loop (ddns_t *ctx);
+int ddns_get_tcp_force(const ddns_info_t *info);
 
 int common_request (ddns_t       *ctx,   ddns_info_t *info, ddns_alias_t *alias);
 int common_response(http_trans_t *trans, ddns_info_t *info, ddns_alias_t *alias);
