@@ -1,7 +1,7 @@
 /* Error code definitions
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -54,6 +54,8 @@
 #define RC_DDNS_RSP_NOHOST              47
 #define RC_DDNS_RSP_NOTOK               48
 #define RC_DDNS_RSP_RETRY_LATER         49
+#define RC_DDNS_RSP_AUTH_FAIL           50
+#define RC_DDNS_RSP_TOO_FREQUENT        51
 
 #define RC_OS_INVALID_IP_ADDRESS        61
 #define RC_OS_FORK_FAILURE              62
