@@ -1,7 +1,7 @@
 /* Reads and updates cache files
  *
  * Copyright (C) 2003-2004  Narcis Ilisei <inarcis2002@hotpop.com>
- * Copyright (C) 2010-2017  Joachim Nilsson <troglobit@gmail.com>
+ * Copyright (C) 2010-2021  Joachim Wiberg <troglobit@gmail.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,10 +31,11 @@
  * once it has read the IP and the modification time.
  */
 
-#include <resolv.h>
-#include <sys/stat.h>
 #include <time.h>
 #include <unistd.h>
+#include <netinet/in.h>
+#include <resolv.h>
+#include <sys/stat.h>
 
 #include "ddns.h"
 #include "cache.h"
@@ -43,9 +44,9 @@ extern ddns_info_t *conf_info_iterator(int first);
 
 static int nslookup(ddns_alias_t *alias)
 {
-	int error;
-	struct addrinfo hints;
 	struct addrinfo *result;
+	struct addrinfo hints;
+	int error;
 
 	memset(&hints, 0, sizeof(struct addrinfo));
 	hints.ai_family = AF_INET;	/* IPv4 */
@@ -74,18 +75,19 @@ static int nslookup(ddns_alias_t *alias)
 	return 1;
 }
 
-static void read_one(ddns_alias_t *alias, int nonslookup)
+static void read_one(ddns_alias_t *alias, const char *name, int nonslookup)
 {
-	FILE *fp;
 	char path[256];
+	FILE *fp;
 
 	alias->last_update = 0;
 	memset(alias->address, 0, sizeof(alias->address));
 
-	cache_file(alias->name, path, sizeof(path));
+	cache_file(alias->name, name, path, sizeof(path));
 	fp = fopen(path, "r");
 	if (!fp) {
-		if (nonslookup)
+		/* Exception for dnsomatic's special global hostname */
+		if (nonslookup || !strncmp(alias->name, "all.dnsomatic.com", sizeof(alias->name)))
 			return;
 
 		/* Try a DNS lookup of our last known IP#. */
@@ -109,12 +111,13 @@ static void read_one(ddns_alias_t *alias, int nonslookup)
 	}
 }
 
-char *cache_file(char *name, char *buf, size_t len)
+char *cache_file(char *name, const char *sysname, char *buf, size_t len)
 {
 	if (!buf || !name)
 		return NULL;
 
-	snprintf(buf, len, "%s/%s.cache", cache_dir, name);
+	if (snprintf(buf, len, "%s/%s-%s.cache", cache_dir, sysname, name) >= (int)len)
+		logit(LOG_WARNING, "Too long name for buffer: '%s/' + '%s' + '.cache'", cache_dir, name);
 
 	return buf;
 }
@@ -128,7 +131,6 @@ char *cache_file(char *name, char *buf, size_t len)
  */
 int read_cache_file(ddns_t *ctx)
 {
-	size_t j;
 	ddns_info_t *info;
 
 	/*
@@ -143,13 +145,26 @@ int read_cache_file(ddns_t *ctx)
 
 	info = conf_info_iterator(1);
 	while (info) {
-		int nonslookup;
+		/* XXX: Possibly move this exception to each plugin */
+		const char *except[] = {
+			"ipv6tb@he.net",
+			"default@tunnelbroker.net"
+		};
+		const char *name = info->system->name;
+		size_t i, j;
+		int nonslookup = 0;
+
+		/* Exceptions -- no name to lookup */
+		for (i = 0; i < NELEMS(except); i++) {
+			if (!strncmp(name, except[i], strlen(name))) {
+				nonslookup = 1;
+				break;
+			}
+		}
 
-		/* Exception for tunnelbroker.net - no name to lookup */
-		nonslookup = !strcmp(info->system->name, "ipv6tb@he.net");
 // XXX: TODO better plugin identifiction here
 		for (j = 0; j < info->alias_count; j++)
-			read_one(&info->alias[j], nonslookup);
+			read_one(&info->alias[j], name, nonslookup);
 
 		info = conf_info_iterator(0);
 	}
@@ -161,15 +176,20 @@ int read_cache_file(ddns_t *ctx)
  * Update cache with new IP
  * /var/cache/inadyn/my.server.name.cache { LAST-IPADDR } MTIME
  */
-int write_cache_file(ddns_alias_t *alias)
+int write_cache_file(ddns_alias_t *alias, const char *name)
 {
+	char path[256];	
 	FILE *fp;
-	char path[256];
 
-	cache_file(alias->name, path, sizeof(path));
+	cache_file(alias->name, name, path, sizeof(path));
 	fp = fopen(path, "w");
 	if (fp) {
-		logit(LOG_NOTICE, "Updating cache for %s", alias->name);
+		const char *prefix = "ipv6";
+
+		if (!strncmp(name, prefix, strlen(prefix)))
+			logit(LOG_NOTICE, "Updating IPv6 cache for %s", alias->name);
+		else
+			logit(LOG_NOTICE, "Updating IPv4 cache for %s", alias->name);
 		fprintf(fp, "%s", alias->address);
 		fclose(fp);
 
