@@ -211,7 +211,8 @@ static int getserver(const char *server, ddns_name_t *name)
 		if (-1 == name->port)
 			name->port = HTTP_DEFAULT_PORT;
 	} else {
-		name->port = HTTP_DEFAULT_PORT;
+		/* Let *ssl.c and tcp.c figure it out later */
+		name->port = 0;
 	}
 
 	strlcpy(name->name, str, sizeof(name->name));
@@ -231,6 +232,15 @@ static int cfg_getserver(cfg_t *cfg, char *server, ddns_name_t *name)
 	return getserver(str, name);
 }
 
+static int cfg_getbool_or_default(cfg_t *cfg, const char *optname, int unset_value)
+{
+	/* Retrieve the boolean if there is any value configured in the provider's config for that option */
+	if (cfg_size(cfg, optname) > 0)
+		return cfg_getbool(cfg, optname);
+
+	return unset_value;
+}
+
 #if 0
 /* TODO: Move to a separate file */
 #define string_startswith(string, prefix) strncasecmp(string, prefix, strlen(prefix)) == 0
@@ -346,7 +356,7 @@ static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 
 	info->wildcard = cfg_getbool(cfg, "wildcard");
 	info->ttl = cfg_getint(cfg, "ttl");
-	info->proxied = cfg_getbool(cfg, "proxied");
+	info->proxied = cfg_getbool_or_default(cfg, "proxied", -1);
 	info->ssl_enabled = cfg_getbool(cfg, "ssl");
 	str = cfg_getstr(cfg, "username");
 	if (str && strlen(str) <= sizeof(info->creds.username))
@@ -354,6 +364,7 @@ static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 	str = cfg_getstr(cfg, "password");
 	if (str && strlen(str) <= sizeof(info->creds.password))
 		strlcpy(info->creds.password, str, sizeof(info->creds.password));
+	info->ifname = cfg_getstr(cfg, "iface");
 
 	for (j = 0; j < cfg_size(cfg, "hostname"); j++) {
 		size_t pos = info->alias_count;
@@ -443,8 +454,8 @@ static int set_provider_opts(cfg_t *cfg, ddns_info_t *info, int custom)
 
 		if (!strcasecmp(info->checkip_name.name, "default")) {
 			strlcpy(info->checkip_name.name, DDNS_MY_IP_SERVER, sizeof(info->checkip_name.name));
-			strlcpy(info->checkip_url, "/", sizeof(info->checkip_url));
-			/* let local checkip-ssl decide HTTP/HTTPS for ipify */
+			strlcpy(info->checkip_url, DDNS_MY_CHECKIP_URL, sizeof(info->checkip_url));
+			info->checkip_ssl = DDNS_MY_IP_SSL;
 		}
 
 		/*
@@ -542,8 +553,9 @@ cfg_t *conf_parse_file(char *file, ddns_t *ctx)
 		CFG_STR_LIST("alias",        NULL, CFGF_DEPRECATED),
 		CFG_BOOL    ("ssl",          cfg_true, CFGF_NONE),
 		CFG_BOOL    ("wildcard",     cfg_false, CFGF_NONE),
-		CFG_INT     ("ttl",          -1, CFGF_NODEFAULT),
-		CFG_BOOL    ("proxied",      cfg_false, CFGF_NONE),
+		CFG_INT     ("ttl",          -1, CFGF_NONE),
+		CFG_BOOL    ("proxied",      cfg_false, CFGF_NODEFAULT),
+		CFG_STR     ("iface",          NULL, CFGF_NONE), /* interface name */
 		CFG_STR     ("checkip-server", NULL, CFGF_NONE), /* Syntax:  name:port */
 		CFG_STR     ("checkip-path",   NULL, CFGF_NONE), /* Default: "/" */
 		CFG_BOOL    ("checkip-ssl",    cfg_true, CFGF_NONE),
@@ -561,8 +573,9 @@ cfg_t *conf_parse_file(char *file, ddns_t *ctx)
 		CFG_STR_LIST("alias",        NULL, CFGF_DEPRECATED),
 		CFG_BOOL    ("ssl",          cfg_true, CFGF_NONE),
 		CFG_BOOL    ("wildcard",     cfg_false, CFGF_NONE),
-		CFG_INT     ("ttl",          -1, CFGF_NODEFAULT),
+		CFG_INT     ("ttl",          -1, CFGF_NONE),
 		CFG_BOOL    ("proxied",      cfg_false, CFGF_NONE),
+		CFG_STR     ("iface",          NULL, CFGF_NONE), /* interface name */
 		CFG_STR     ("checkip-server", NULL, CFGF_NONE), /* Syntax:  name:port */
 		CFG_STR     ("checkip-path",   NULL, CFGF_NONE), /* Default: "/" */
 		CFG_BOOL    ("checkip-ssl",    cfg_true, CFGF_NONE),
