@@ -136,7 +136,7 @@ static int server_transaction(ddns_t *ctx, ddns_info_t *provider)
 
 	client = &provider->checkip;
 	client->ssl_enabled = provider->checkip_ssl;
-	DO(http_init(client, "Checking for IP# change", strstr(provider->system->name, "ipv6") ? TCP_FORCE_IPV6 : TCP_FORCE_IPV4));
+	DO(http_init(client, "Checking for IP# change", ddns_get_tcp_force(provider)));
 
 	/* Prepare request for IP server */
 	memset(ctx->work_buf, 0, ctx->work_buflen);
@@ -192,9 +192,8 @@ static int is_address_valid(int family, const char *host)
 	size_t i;
 
 	for (i = 0; i < NELEMS(except); i++) {
-		if (!strncmp(host, except[i], strlen(host))) {
+		if (!strncmp(host, except[i], strlen(host)))
 			return 0;
-		}
 	}
 
 	if (!verify_addr) {
@@ -335,7 +334,7 @@ static int parse_my_address(char *buffer, char *address, size_t len)
 
 static int get_address_remote(ddns_t *ctx, ddns_info_t *info, char *address, size_t len)
 {
-	if (!info->server_url[0])
+	if (!info->checkip_name.name[0])
 		return 1;
 
 	DO(server_transaction(ctx, info));
@@ -352,9 +351,6 @@ static int get_address_remote(ddns_t *ctx, ddns_info_t *info, char *address, siz
 
 static int get_address_cmd(ddns_t *ctx, ddns_info_t *info, char *address, size_t len)
 {
-	if (!info->checkip_cmd || !info->checkip_cmd[0])
-		return 1;
-
 	DO(shell_transaction(ctx, info, info->checkip_cmd));
 	logit(LOG_DEBUG, "Command response:");
 	logit(LOG_DEBUG, "%s", ctx->work_buf);
@@ -403,9 +399,6 @@ static int get_address_iface(ddns_t *ctx, const char *ifname, char *address, siz
 	char *ptr, trailer[IFNAMSIZ + 2];
 	struct ifaddrs *ifaddr, *ifa;
 
-	if (!ifname || !ifname[0])
-		return 1;
-
 	/* Trailer to strip, if set by getnameinfo() */
 	snprintf(trailer, sizeof(trailer), "%%%s", ifname);
 
@@ -467,17 +460,22 @@ static int get_address_backend(ddns_t *ctx, ddns_info_t *info, char *address, si
 	logit(LOG_DEBUG, "Get address for %s", info->system->name);
 	memset(address, 0, len);
 
-	if (!get_address_cmd   (ctx, info, address, len))
-		return 0;
-
-	/* Check info specific interface */
-	if (!get_address_iface (ctx, info->ifname, address, len))
-		return 0;
-
-	/* Check the global interface */
-	if (!get_address_iface (ctx, iface, address, len))
-		return 0;
+	if (info->checkip_cmd && info->checkip_cmd[0]) {
+		/* Get address from command */
+		return get_address_cmd(ctx, info, address, len);
+	}
+	
+	if (info->ifname && info->ifname[0]) {
+		/* Get address from specific interface */
+		return get_address_iface(ctx, info->ifname, address, len);
+	}
+	
+	if (iface && iface[0]) {
+		/* Get address from global interface */
+		return get_address_iface(ctx, iface, address, len);
+	}
 
+	/* Get address from remote service */
 	if (!get_address_remote(ctx, info, address, len))
 		return 0;
 
@@ -619,7 +617,7 @@ static int send_update(ddns_t *ctx, ddns_info_t *info, ddns_alias_t *alias, int
 		DO(info->system->setup(ctx, info, alias));
 
 	client->ssl_enabled = info->ssl_enabled;
-	rc = http_init(client, "Sending IP# update to DDNS server", strstr(info->system->name, "ipv6") ? TCP_FORCE_IPV6 : TCP_FORCE_IPV4);
+	rc = http_init(client, "Sending IP# update to DDNS server", ddns_get_tcp_force(info));
 	if (rc) {
 		/* Update failed, force update again in ctx->cmd_check_period seconds */
 		alias->force_addr_update = 1;
@@ -992,6 +990,7 @@ int ddns_main_loop(ddns_t *ctx)
 			info = conf_info_iterator(1);
 			while (info) {
 				size_t i;
+
 				for (i = 0; i < info->alias_count; i++) {
 					ddns_alias_t *alias = &info->alias[i];
 					alias->force_addr_update = 1;
@@ -1042,6 +1041,7 @@ int ddns_main_loop(ddns_t *ctx)
 			info = conf_info_iterator(1);
 			while (info) {
 				size_t i;
+
 				for (i = 0; i < info->alias_count; i++) {
 					ddns_alias_t *alias = &info->alias[i];
 					alias->force_addr_update = 1;
@@ -1065,6 +1065,17 @@ int ddns_main_loop(ddns_t *ctx)
 	return rc;
 }
 
+int ddns_get_tcp_force(const ddns_info_t *info)
+{
+	const char* name = info->system->name;
+	const char *prefix = "ipv6";
+
+	if (!strncmp(name, prefix, strlen(prefix)))
+		return TCP_FORCE_IPV6;
+
+	return TCP_FORCE_IPV4;
+}
+
 /**
  * Local Variables:
  *  indent-tabs-mode: t
